package com.sinoqixin.bgdrocketmq.service.handle;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.client.consumer.listener.ConsumeOrderlyStatus;
import org.apache.rocketmq.client.consumer.listener.MessageListenerConcurrently;
import org.apache.rocketmq.client.consumer.listener.MessageListenerOrderly;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.common.message.MessageExt;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

@Slf4j
public class MqDataHandleFactory implements Runnable {

    private final Map<String, Map<String, MqDataHandle>> handleMap;

    private final DefaultMQPushConsumer consumer;

    private final Boolean messageOrder;

    private String CUR_TOPIC = StringUtils.EMPTY; //当前消费主题

    private String CUR_TAB_NAME = StringUtils.EMPTY; //当前消费主题tab

    private long CUR_QUEUE_OFFSET = 0; //当前消费主题偏移量

    public MqDataHandleFactory(Map<String, Map<String, MqDataHandle>> handleMap, DefaultMQPushConsumer consumer, Boolean messageOrder) {
        this.handleMap = handleMap;
        this.consumer = consumer;
        this.messageOrder = messageOrder;
    }

    @Override
    public void run() {
        String curThreadName = Thread.currentThread().getName();
        log.info("rocketMq【{}】订阅主题为：{}", curThreadName, Arrays.toString(this.handleMap.keySet().toArray()));
        if (this.handleMap.isEmpty()) {
            log.info("rocketMq【{}】订阅主题为空。", curThreadName);
            return;
        }
        try {
            for (String key : this.handleMap.keySet()) {
                String tab = StringUtils.EMPTY;
                for (String tabKey : this.handleMap.get(key).keySet()) {
                    tab = tabKey;
                    break;
                }
                if (StringUtils.isBlank(tab)) tab = "*";
                this.consumer.subscribe(key, tab);
            }
            //是否需要顺序执行
            if (messageOrder == null || !messageOrder) {
                consumer.registerMessageListener((MessageListenerConcurrently) (list, consumeConcurrentlyContext) -> {
                    log.info("rocketMq 进入异步消息监听");
                    handleData(list);
                    // 返回消费状态
                    return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                });
            } else {
                //注册顺序消息监听器
                consumer.registerMessageListener((MessageListenerOrderly) (list, consumeOrderlyContext) -> {
                    log.info("rocketMq 进入顺序消息监听");
                    handleData(list);
                    // 返回消费状态
                    return ConsumeOrderlyStatus.SUCCESS;
                });
            }
            log.info("rocketMq 注册监听成功");
            consumer.start();
        } catch (MQClientException e) {
            log.error("", e);
            log.error("rocketMq 监听失败，失败原因：{}", e.getErrorMessage());
        }
    }

    public void handleData(List<MessageExt> list) {
        if (list.isEmpty()) return;
        Integer queueId = null;
        long pollTime = System.currentTimeMillis();
        for (MessageExt messageExt : list) {
            String tags = messageExt.getTags();
            String topic = messageExt.getTopic();
            queueId = messageExt.getQueueId();
            MqDataHandle mqDataHandle = getMqDataHandle(handleMap, topic);
            long queueOffset = messageExt.getQueueOffset();
            String msg = new String(messageExt.getBody());
            if (!CUR_TOPIC.equals(topic)) {
                log.info("rocketMq 切换主题：{}，tab：{}，第一条数据：{}", topic, tags, msg);
                CUR_TOPIC = topic;
                CUR_TAB_NAME = tags;
            }
            CUR_QUEUE_OFFSET = queueOffset;
            if (mqDataHandle == null) continue;
            try {
                log.info("rocket获取数据主题 ===>：{}，tab：{}，queueId:{}，offset：{}, 数据：{}", topic, tags, queueId, queueOffset, msg);
                mqDataHandle.handleData(msg);
            } catch (Exception e) {
                log.error("", e);
                log.info("rocketMq 处理数据异常：{}，poll数据为：{}", e.getMessage(), msg);
            }
        }
        long finalTime = System.currentTimeMillis();
        log.info("rocketMq：主题【最后一条】：{}，tab：{}，拉取数据：{}条，用时：{}s，queueId【最后一条】：{}，offset【最后一条】：{}", CUR_TOPIC, CUR_TAB_NAME, list.size(), (finalTime - pollTime) / 1000.0, queueId, CUR_QUEUE_OFFSET);
    }

    public MqDataHandle getMqDataHandle(Map<String, Map<String, MqDataHandle>> handleMap, String topic) {
        MqDataHandle mqDataHandle = null;
        Map<String, MqDataHandle> stringMqDataHandleMap = handleMap.get(topic);
        for (String key : stringMqDataHandleMap.keySet()) {
            mqDataHandle = stringMqDataHandleMap.get(key);
            break;
        }
        return mqDataHandle;
    }
}
